package CDS::EpicsVariable;

@ISA = {'Exporter'};

# Represents a single epics variable needed by some model part
# Parts can return a list of these in getEpicsVariables
# instead of writing out to a file with printHeaderStruct and
# printEpics

#  create a new EpicsVariable object
#  args:
#  1 - EPICS name, without leading "X1:" where X1 is the IFO code
#  2 - C name
#  3 - C type, one of ["int", "uint", "double"]
#  4 - Epics type, one of ["ao", "ai", "bo", "bi"]
#  5 - Precision - epics variable PREC field
# returns an object reference with values
# epics_name
# c_name
# c_type
# epics_type
# precision

my @valid_ctypes = qw(int uint double);

my @valid_epics_types = qw(ao ai bo bi);

# the prefix of the epics line has to announce
# whether the variable is an INVARIABLE or an OUTVARIABLE
my %epics_class = (
    'ao' => 'OUTVARIABLE',
    'bo' => 'OUTVARIABLE',
    'ai' => 'INVARIABLE',
    'bi' => 'INVARIABLE',
);

sub new {
	my $epics_name = shift;
	my $c_name = shift;
	my $c_type = shift;
	my $epics_type = shift;
	my $precision = shift;
    my $create_mask = shift;

	$good_ctype = 0;
	for( @valid_ctypes) {
		if ($c_type eq $_) {
			$good_ctype = 1;
			last;
		}
	}
	die "EPICS variable '$epics_name': Invalid C type '$c_type'." unless $good_ctype;

	$good_epics_type = 0;
	for( @valid_epics_types) {
		if ($epics_type eq $_) {
			$good_epics_type = 1;
			last;
		}
	}

	die "EPICS variable '$epics_name': Invalid EPICS type '$epics_type'." unless $good_epics_type;

	my $self = {
        "epics_name"  => $epics_name,
        "c_name"      => $c_name,
        "c_type"      => $c_type,
        "epics_type" => $epics_type,
        "precision"   => $precision,
        "create_mask" => $create_mask,
	};

	bless $self;

	return $self;
}


# return a C declarations for the variable
sub getCDeclaration {
	my $self = shift;

	return "$$self{c_type} $$self{c_name};";
}

# return a C mask declarations for the variable
sub getCMaskDeclaration {
    my $self = shift;

    if ($self{create_mask}) {
        return "$$self{c_type} $$self{c_name}_mask;";
    }
    else {
        return "";
    }
}

# return a list of epics DB entries for the variable
sub getEpicsEntries {
	my $self = shift;

    $prec = $$self{precision};

    if ($prec) {
        $prec_str = " field(PREC,\"$prec\")";
    }
    else
    {
        $prec_str = "";
    }

    $class = $epics_class{$$self{epics_type}};

    die "Epics var '$$self{epics_name}': couldn't find class for epics type '$$self{epics_type}'" unless $class;

	return ("$class $$self{epics_name} $::systemName\.$$self{c_name} $$self{c_type} $$self{epics_type} 0" . $prec_str, );
}

# returns true if variable is an integer type
sub isInteger {
    my $self = shift;

    return ( $$self{c_type} eq 'int' || $$self{c_type} eq 'uint' );
}

# returns true if variable is an integer type
sub isDouble {
    my $self = shift;

    return ( $$self{c_type} eq 'double');
}